@@ -91,6 +91,9 @@ class HostPythonConfiguration:
     def __init__(self):
         """ Initialise the configuration. """
 
+        self.platform = sys.platform
+        self.version = sys.hexversion >> 8
+
         if hasattr(sysconfig, 'get_path'):
             # The modern API.
             self.inc_dir = sysconfig.get_path('include')
@@ -102,12 +105,10 @@ class HostPythonConfiguration:
 
         if sys.platform == 'win32':
             self.data_dir = sys.prefix
-
+            self.lib_dir = sys.prefix + '\\libs'
         else:
             self.data_dir = sys.prefix + '/share'
-
-        # This is only used on Windows.
-        self.lib_dir = sys.prefix + '/libs'
+            self.lib_dir = sys.prefix + '/lib'
 
 
 class TargetQtConfiguration:
@@ -148,23 +149,40 @@ class TargetConfiguration:
 
         # Values based on the host Python configuration.
         py_config = HostPythonConfiguration()
-        self.module_dir = os.path.join(py_config.module_dir, 'PyQt4')
-        self.pyqt_sip_dir = os.path.join(py_config.data_dir, 'sip', 'PyQt4')
-        self.qsci_sip_dir = self.pyqt_sip_dir
+        self.py_module_dir = py_config.module_dir
         self.py_inc_dir = py_config.inc_dir
-        self.sip_inc_dir = py_config.inc_dir
         self.py_lib_dir = py_config.lib_dir
+        self.py_platform = py_config.platform
+        self.py_sip_dir = os.path.join(py_config.data_dir, 'sip')
+        self.py_version = py_config.version
+        self.sip_inc_dir = py_config.inc_dir
+
+        # The default qmake spec.
+        if self.py_platform == 'win32':
+            if self.py_version >= 0x030300:
+                self.qmake_spec = 'win32-msvc2010'
+            elif self.py_version >= 0x020600:
+                self.qmake_spec = 'win32-msvc2008'
+            elif self.py_version >= 0x020400:
+                self.qmake_spec = 'win32-msvc.net'
+            else:
+                self.qmake_spec = 'win32-msvc'
+        else:
+            # Use the Qt default.  (We may update it for MacOS/X later.)
+            self.qmake_spec = ''
 
         # Remaining values.
-        self.platform = sys.platform
-        self.py_version = sys.hexversion >> 8
         self.pyqt_sip_flags = ''
         self.pyqt_version = ''
         self.qmake = self._find_exe('qmake')
         self.sip = self._find_exe('sip')
 
-        self.prot_is_public = (self.platform.startswith('linux') or self.platform == 'darwin')
-        self.qscintilla_is_dll = (self.platform == 'win32')
+        self.prot_is_public = (self.py_platform.startswith('linux') or self.py_platform == 'darwin')
+        self.qscintilla_is_dll = (self.py_platform == 'win32')
+
+        self.module_dir = os.path.join(py_config.module_dir, 'PyQt4')
+        self.pyqt_sip_dir = os.path.join(self.py_sip_dir, 'PyQt4')
+        self.qsci_sip_dir = self.pyqt_sip_dir
 
     def from_configuration_file(self, config_file):
         """ Initialise the configuration with values from a file.  config_file
@@ -209,15 +227,30 @@ class TargetConfiguration:
 
         cfg.close()
 
-    def from_introspection(self):
-        """ Initialise the configuration by introspecting the system. """
+    def from_introspection(self, pyqt_package):
+        """ Initialise the configuration by introspecting the system.
+        pyqt_package is the name of the PyQt package we are building against.
+        """
 
-        from PyQt4 import QtCore
+        if pyqt_package == 'PyQt5':
+            try:
+                from PyQt5 import QtCore
+            except ImportError:
+                error(
+                        "Unable to import PyQt5.QtCore. Make sure PyQt5 is "
+                        "installed.")
+        else:
+            try:
+                from PyQt4 import QtCore
+            except ImportError:
+                error(
+                        "Unable to import PyQt4.QtCore. Make sure PyQt4 is "
+                        "installed.")
 
         inform("PyQt %s is being used." % QtCore.PYQT_VERSION_STR)
         inform("Qt %s is being used." % QtCore.QT_VERSION_STR)
 
-        # See if we have a PyQt4 that embeds its configuration.
+        # See if we have a PyQt that embeds its configuration.
         try:
             pyqt_config = QtCore.PYQT_CONFIGURATION
         except AttributeError:
@@ -237,12 +270,11 @@ class TargetConfiguration:
 
         self.pyqt_sip_flags = pyqt_config['sip_flags']
 
-    def override_defaults(self, opts):
-        """ Override the default from the command line.  opts are the command
-        line options.
+    def get_qt_configuration(self, opts):
+        """ Get the Qt configuration that can be extracted from qmake.  opts
+        are the command line options.
         """
 
-        # Values from the target Qt configuration.
         try:
             qmake = opts.qmake
         except AttributeError:
@@ -264,31 +296,48 @@ class TargetConfiguration:
                         "or use the --qmake argument to explicitly specify a "
                         "working Qt qmake.")
 
+        # Query qmake.
         qt_config = TargetQtConfiguration(self.qmake)
+
+        # The binary MacOS/X Qt installer defaults to XCode.  If this is what
+        # we might have then use macx-clang (Qt v5) or macx-g++ (Qt v4).
+        if sys.platform == 'darwin':
+            try:
+                # Qt v5.
+                if qt_config.QMAKE_SPEC == 'macx-xcode':
+                    # This will exist (and we can't check anyway).
+                    self.qmake_spec = 'macx-clang'
+                else:
+                    # No need to explicitly name the default.
+                    self.qmake_spec = ''
+            except AttributeError:
+                # Qt v4.
+                self.qmake_spec = 'macx-g++'
+
         self.api_dir = qt_config.QT_INSTALL_DATA
         self.qsci_inc_dir = qt_config.QT_INSTALL_HEADERS
         self.qsci_lib_dir = qt_config.QT_INSTALL_LIBS
 
-        # Override from the command line.
+    def override_defaults(self, opts):
+        """ Override the defaults from the command line.  opts are the command
+        line options.
+        """
+
         if opts.apidir is not None:
             self.api_dir = opts.apidir
 
-        inform("The QScintilla API file will be installed in %s." %
-                os.path.join(self.api_dir, 'api', 'python'))
-
         if opts.destdir is not None:
             self.module_dir = opts.destdir
+        else:
+            self.module_dir = os.path.join(self.py_module_dir,
+                    opts.pyqt_package)
 
-        inform("The QScintilla module will be installed in %s." %
-                self.module_dir)
+        if opts.qmakespec is not None:
+            self.qmake_spec = opts.qmakespec
 
         if opts.prot_is_public is not None:
             self.prot_is_public = opts.prot_is_public
 
-        if self.prot_is_public:
-            inform("The QScintilla module is being built with 'protected' "
-                    "redefined as 'public'.")
-
         if opts.qsci_inc_dir is not None:
             self.qsci_inc_dir = opts.qsci_inc_dir
 
@@ -300,12 +349,14 @@ class TargetConfiguration:
 
         if opts.pyqt_sip_dir is not None:
             self.pyqt_sip_dir = opts.pyqt_sip_dir
+        else:
+            self.pyqt_sip_dir = os.path.join(self.py_sip_dir,
+                    opts.pyqt_package)
 
         if opts.qsci_sip_dir is not None:
             self.qsci_sip_dir = opts.qsci_sip_dir
-
-        inform("The QScintilla .sip files will be installed in %s." %
-                self.qsci_sip_dir)
+        else:
+            self.qsci_sip_dir = self.pyqt_sip_dir
 
         if opts.sip is not None:
             self.sip = opts.sip
@@ -339,7 +390,7 @@ def create_optparser(target_config):
     configuration containing default values.
     """
 
-    def store_abspath_file(option, opt_str, value, parser):
+    def store_abspath(option, opt_str, value, parser):
         setattr(parser.values, option.dest, os.path.abspath(value))
 
     def store_abspath_dir(option, opt_str, value, parser):
@@ -353,18 +404,21 @@ def create_optparser(target_config):
         setattr(parser.values, option.dest, os.path.abspath(value))
 
     p = optparse.OptionParser(usage="python %prog [options]",
-            version="2.7.1")
+            version="2.7.2")
 
+    p.add_option("--spec", dest='qmakespec', default=None, action='store',
+            metavar="SPEC",
+            help="pass -spec SPEC to qmake [default: %s]" % "don't pass -spec" if target_config.qmake_spec == '' else target_config.qmake_spec)
     p.add_option("--apidir", "-a", dest='apidir', type='string', default=None,
-            action='callback', callback=store_abspath_file, metavar="DIR", 
+            action='callback', callback=store_abspath, metavar="DIR", 
             help="the QScintilla API file will be installed in DIR [default: "
                     "QT_INSTALL_DATA/qsci]")
     p.add_option("--configuration", dest='config_file', type='string',
-            default=None, action='callback', callback=store_abspath_file,
+            default=None, action='callback', callback=store_abspath,
             metavar="FILE",
             help="FILE defines the target configuration")
     p.add_option("--destdir", "-d", dest='destdir', type='string',
-            default=None, action='callback', callback=store_abspath_file,
+            default=None, action='callback', callback=store_abspath,
             metavar="DIR",
             help="install the QScintilla module in DIR [default: "
                     "%s]" % target_config.module_dir)
@@ -375,6 +429,10 @@ def create_optparser(target_config):
     p.add_option("--protected-not-public", dest='prot_is_public',
             action='store_false',
             help="disable building with 'protected' redefined as 'public'")
+    p.add_option("--pyqt", dest='pyqt_package', type='choice',
+            choices=['PyQt4', 'PyQt5'], default='PyQt4', action='store',
+            metavar="PyQtn",
+            help="configure for PyQt4 or PyQt5 [default: PyQt4]")
 
     if sys.platform != 'win32':
         p.add_option("--qmake", "-q", dest='qmake', type='string',
@@ -442,6 +500,26 @@ def create_optparser(target_config):
     return p
 
 
+def inform_user(target_config):
+    """ Tell the user the values that are going to be used.  target_config is
+    the target configuration.
+    """
+
+    inform("The sip executable is %s." % target_config.sip)
+
+    inform("The QScintilla module will be installed in %s." % target_config.module_dir)
+
+    if target_config.prot_is_public:
+        inform("The QScintilla module is being built with 'protected' "
+                "redefined as 'public'.")
+
+    inform("The QScintilla .sip files will be installed in %s." %
+            target_config.qsci_sip_dir)
+
+    inform("The QScintilla API file will be installed in %s." %
+                os.path.join(target_config.api_dir, 'api', 'python'))
+
+
 def check_qscintilla(target_config):
     """ See if QScintilla can be found and what its version is.  target_config
     is the target configuration.
@@ -466,8 +544,8 @@ def check_qscintilla(target_config):
     # Because we include the Python bindings with the C++ code we can
     # reasonably force the same version to be used and not bother about
     # versioning.
-    if sciversstr != '2.7.1':
-        error("QScintilla %s is being used but the Python bindings v2.7.1 are being built. Please use matching versions." % sciversstr)
+    if sciversstr != '2.7.2':
+        error("QScintilla %s is being used but the Python bindings 2.7.2 are being built. Please use matching versions." % sciversstr)
 
     inform("QScintilla %s is being used." % sciversstr)
 
@@ -542,7 +620,10 @@ def generate_code(target_config, opts):
     argv.append('-c')
     argv.append('.')
 
-    argv.append('sip/qscimod4.sip')
+    if opts.pyqt_package == 'PyQt5':
+        argv.append('sip/qscimod5.sip')
+    else:
+        argv.append('sip/qscimod4.sip')
 
     rc = os.spawnv(os.P_WAIT, target_config.sip, argv)
     if rc != 0:
@@ -554,7 +635,13 @@ def generate_code(target_config, opts):
     # Generate the Makefile.
     inform("Creating the Makefile for the Qsci module...")
 
-    rc = os.spawnl(os.P_WAIT, target_config.qmake, 'qmake', 'Qsci.pro')
+    qmake_args = ['qmake']
+    if target_config.qmake_spec != '':
+        qmake_args.append('-spec')
+        qmake_args.append(target_config.qmake_spec)
+    qmake_args.append('Qsci.pro')
+
+    rc = os.spawnv(os.P_WAIT, target_config.qmake, qmake_args)
     if rc != 0:
         error("%s returned exit code %d." % (target_config.qmake, rc))
 
@@ -619,10 +706,10 @@ INSTALLS += sip
     pro.write('\n')
 
     # These optimisations could apply to other platforms.
-    if target_config.platform == 'darwin':
+    if target_config.py_platform == 'darwin':
         pro.write('QMAKE_CXXFLAGS += -fno-exceptions\n')
 
-    if target_config.platform.startswith('linux'):
+    if target_config.py_platform.startswith('linux'):
         pro.write('QMAKE_CXXFLAGS += -fno-exceptions\n')
 
         if not opts.static:
@@ -757,11 +844,14 @@ def main(argv):
         p.print_help()
         sys.exit(2)
 
+    # Query qmake for the basic configuration information.
+    target_config.get_qt_configuration(opts)
+
     # Update the target configuration.
     if opts.config_file is not None:
         target_config.from_configuration_file(opts.config_file)
     else:
-        target_config.from_introspection()
+        target_config.from_introspection(opts.pyqt_package)
 
     target_config.override_defaults(opts)
 
@@ -771,6 +861,9 @@ def main(argv):
     # Check for QScintilla.
     check_qscintilla(target_config)
 
+    # Tell the user what's been found.
+    inform_user(target_config)
+
     # Generate the code.
     generate_code(target_config, opts)
 
@@ -779,7 +872,7 @@ def main(argv):
 # The script starts here.
 ###############################################################################
 
-if __name__ == "__main__":
+if __name__ == '__main__':
     try:
         main(sys.argv)
     except SystemExit:
