@@ -109,7 +109,7 @@ void Font::Create(const FontParameters &fp)
         strategy = QFont::PreferDefault;
     }
 
-#if defined(Q_WS_MAC)
+#if defined(Q_OS_MAC)
 #if QT_VERSION >= 0x040700
     strategy = static_cast<QFont::StyleStrategy>(strategy | QFont::ForceIntegerMetrics);
 #else
@@ -228,6 +228,7 @@ private:
     void drawRect(const PRectangle &rc);
     void drawText(const PRectangle &rc, Font &font_, XYPOSITION ybase,
             const char *s, int len, ColourDesired fore);
+    static QFont convertQFont(Font &font);
     QFontMetrics metrics(Font &font_);
     QString convertText(const char *s, int len);
     static QColor convertQColor(const ColourDesired &col,
@@ -553,8 +554,8 @@ void SurfaceImpl::DrawRGBAImage(PRectangle rc, int width, int height,
 void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len,
         XYPOSITION *positions)
 {
-    QFontMetrics fm = metrics(font_);
     QString qs = convertText(s, len);
+    QFontMetrics fm = metrics(font_);
 
     // The position for each byte of a character is the offset from the start
     // where the following character should be drawn.
@@ -564,7 +565,7 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len,
     {
         // We can't just add the individual character widths together because
         // of kerning.
-        XYPOSITION width = fm.width(qs.left(i_char + 1));
+        XYPOSITION width = qRound(fm.width(qs.left(i_char + 1)));
 
         if (unicodeMode)
         {
@@ -640,15 +641,21 @@ void SurfaceImpl::FlushCachedState()
 {
 }
 
+// Return the QFont for a Font.
+QFont SurfaceImpl::convertQFont(Font &font)
+{
+    QFont *f = PFont(font.GetID());
+
+    if (f)
+        return *f;
+
+    return QApplication::font();
+}
+
 // Get the metrics for a font.
 QFontMetrics SurfaceImpl::metrics(Font &font_)
 {
-    QFont *f = PFont(font_.GetID()), fnt;
-
-    if (f)
-        fnt = *f;
-    else
-        fnt = QApplication::font();
+    QFont fnt = convertQFont(font_);
 
     if (painter && painter->isActive())
     {
