@@ -1,6 +1,6 @@
 // This defines the interface to the QsciLexerCPP class.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -53,67 +48,102 @@ public:
     enum {
         //! The default.
         Default = 0,
+        InactiveDefault = Default + 64,
 
         //! A C comment.
         Comment = 1,
+        InactiveComment = Comment + 64,
 
         //! A C++ comment line.
         CommentLine = 2,
+        InactiveCommentLine = CommentLine + 64,
 
         //! A JavaDoc/Doxygen style C comment.
         CommentDoc = 3,
+        InactiveCommentDoc = CommentDoc + 64,
 
         //! A number.
         Number = 4,
+        InactiveNumber = Number + 64,
 
         //! A keyword.
         Keyword = 5,
+        InactiveKeyword = Keyword + 64,
 
         //! A double-quoted string.
         DoubleQuotedString = 6,
+        InactiveDoubleQuotedString = DoubleQuotedString + 64,
 
         //! A single-quoted string.
         SingleQuotedString = 7,
+        InactiveSingleQuotedString = SingleQuotedString + 64,
 
         //! An IDL UUID.
         UUID = 8,
+        InactiveUUID = UUID + 64,
 
         //! A pre-processor block.
         PreProcessor = 9,
+        InactivePreProcessor = PreProcessor + 64,
 
         //! An operator.
         Operator = 10,
+        InactiveOperator = Operator + 64,
 
         //! An identifier
         Identifier = 11,
+        InactiveIdentifier = Identifier + 64,
 
         //! The end of a line where a string is not closed.
         UnclosedString = 12,
+        InactiveUnclosedString = UnclosedString + 64,
 
         //! A C# verbatim string.
         VerbatimString = 13,
+        InactiveVerbatimString = VerbatimString + 64,
 
         //! A JavaScript regular expression.
         Regex = 14,
+        InactiveRegex = Regex + 64,
 
         //! A JavaDoc/Doxygen style C++ comment line.
         CommentLineDoc = 15,
+        InactiveCommentLineDoc = CommentLineDoc + 64,
 
-        //! A keyword defined in keyword set number 2.  The class must
-        //! be sub-classed and re-implement keywords() to make use of
-        //! this style.
+        //! A keyword defined in keyword set number 2.  The class must be
+        //! sub-classed and re-implement keywords() to make use of this style.
         KeywordSet2 = 16,
+        InactiveKeywordSet2 = KeywordSet2 + 64,
 
         //! A JavaDoc/Doxygen keyword.
         CommentDocKeyword = 17,
+        InactiveCommentDocKeyword = CommentDocKeyword + 64,
 
         //! A JavaDoc/Doxygen keyword error.
         CommentDocKeywordError = 18,
+        InactiveCommentDocKeywordError = CommentDocKeywordError + 64,
 
-        //! A global class or typedef defined in keyword set number 4.
-        //! The class must be sub-classed and re-implement keywords()
-        //! to make use of this style.
-        GlobalClass = 19
+        //! A global class or typedef defined in keyword set number 5.  The
+        //! class must be sub-classed and re-implement keywords() to make use
+        //! of this style.
+        GlobalClass = 19,
+        InactiveGlobalClass = GlobalClass + 64,
+
+        //! A C++ raw string.
+        RawString = 20,
+        InactiveRawString = RawString + 20,
+
+        //! A Vala triple-quoted verbatim string.
+        TripleQuotedVerbatimString = 21,
+        InactiveTripleQuotedVerbatimString = TripleQuotedVerbatimString + 21,
+
+        //! A Pike hash-quoted string.
+        HashQuotedString = 22,
+        InactiveHashQuotedString = HashQuotedString + 22,
+
+        //! A pre-processor stream comment.
+        PreProcessorComment = 23,
+        InactivePreProcessorComment = PreProcessorComment + 23,
     };
 
     //! Construct a QsciLexerCPP with parent \a parent and name \a name.  \a
@@ -155,7 +185,7 @@ public:
     //! \internal Returns the style used for braces for brace matching.
     int braceStyle() const;
 
-    //! \internal Returns the string of characters that comprise a word.
+    //! Returns the string of characters that comprise a word.
     const char *wordCharacters() const;
 
     //! Returns the foreground colour of the text for style number \a style.
@@ -175,7 +205,11 @@ public:
     QColor defaultPaper(int style) const;
 
     //! Returns the set of keywords for the keyword set \a set recognised
-    //! by the lexer as a space separated string.
+    //! by the lexer as a space separated string.  Set 1 is normally used for
+    //! primary keywords and identifiers.  Set 2 is normally used for secondary
+    //! keywords and identifiers.  Set 3 is normally used for documentation
+    //! comment keywords.  Set 4 is normally used for global classes and
+    //! typedefs.
     const char *keywords(int set) const;
 
     //! Returns the descriptive name for style number \a style.  If the
@@ -190,28 +224,28 @@ public:
     //! Returns true if "} else {" lines can be folded.
     //!
     //! \sa setFoldAtElse()
-    bool foldAtElse() const;
+    bool foldAtElse() const {return fold_atelse;}
 
     //! Returns true if multi-line comment blocks can be folded.
     //!
     //! \sa setFoldComments()
-    bool foldComments() const;
+    bool foldComments() const {return fold_comments;}
 
     //! Returns true if trailing blank lines are included in a fold block.
     //!
     //! \sa setFoldCompact()
-    bool foldCompact() const;
+    bool foldCompact() const {return fold_compact;}
 
     //! Returns true if preprocessor blocks can be folded.
     //!
     //! \sa setFoldPreprocessor()
-    bool foldPreprocessor() const;
+    bool foldPreprocessor() const {return fold_preproc;}
 
     //! Returns true if preprocessor lines (after the preprocessor
     //! directive) are styled.
     //!
     //! \sa setStylePreprocessor()
-    bool stylePreprocessor() const;
+    bool stylePreprocessor() const {return style_preproc;}
 
     //! If \a allowed is true then '$' characters are allowed in identifier
     //! names.  The default is true.
@@ -222,7 +256,29 @@ public:
     //! Returns true if '$' characters are allowed in identifier names.
     //!
     //! \sa setDollarsAllowed()
-    bool dollarsAllowed() const;
+    bool dollarsAllowed() const {return dollars;}
+
+    //! If \a enabled is true then triple quoted strings are highlighted.  The
+    //! default is false.
+    //!
+    //! \sa highlightTripleQuotedStrings()
+    void setHighlightTripleQuotedStrings(bool enabled);
+
+    //! Returns true if triple quoted strings should be highlighted.
+    //!
+    //! \sa setHighlightTripleQuotedStrings()
+    bool highlightTripleQuotedStrings() const {return highlight_triple;}
+
+    //! If \a enabled is true then hash quoted strings are highlighted.  The
+    //! default is false.
+    //!
+    //! \sa highlightHashQuotedStrings()
+    void setHighlightHashQuotedStrings(bool enabled);
+
+    //! Returns true if hash quoted strings should be highlighted.
+    //!
+    //! \sa setHighlightHashQuotedStrings()
+    bool highlightHashQuotedStrings() const {return highlight_hash;}
 
 public slots:
     //! If \a fold is true then "} else {" lines can be folded.  The
@@ -277,6 +333,8 @@ private:
     void setPreprocProp();
     void setStylePreprocProp();
     void setDollarsProp();
+    void setHighlightTripleProp();
+    void setHighlightHashProp();
 
     bool fold_atelse;
     bool fold_comments;
@@ -284,6 +342,8 @@ private:
     bool fold_preproc;
     bool style_preproc;
     bool dollars;
+    bool highlight_triple;
+    bool highlight_hash;
 
     bool nocase;
 
