@@ -1,6 +1,6 @@
 // This module implements the QsciAPIs class.
 //
-// Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,18 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
-// 
-// This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-// INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-// granted herein.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -136,7 +126,7 @@ QString QsciAPIsPrepared::apiBaseName(const QString &api)
     if (tail >= 0)
         base.truncate(tail);
 
-    return base;
+    return base.simplifyWhiteSpace();
 }
 
 
@@ -313,9 +303,9 @@ void QsciAPIs::clear()
 
 
 // Clear out all API information.
-bool QsciAPIs::load(const QString &fname)
+bool QsciAPIs::load(const QString &filename)
 {
-    QFile f(fname);
+    QFile f(filename);
 
     if (!f.open(IO_ReadOnly))
         return false;
@@ -390,35 +380,47 @@ QStringList QsciAPIs::positionOrigin(const QStringList &context, QString &path)
         int start_new = old_context.count();
         int end_new = new_context.count() - 1;
 
-        QString fixed = *origin;
-        fixed.truncate(origin_len);
-
-        path = fixed;
-
-        while (start_new < end_new)
+        if (start_new == end_new)
         {
-            // Add this word to the current path.
-            path.append(wsep);
-            path.append(new_context[start_new]);
+            path = old_context.join(wsep);
             origin_len = path.length();
-
-            // Skip entries in the current origin that don't match the path.
-            while (origin != prep->raw_apis.end())
-            {
-                // See if the current origin has come to an end.
-                if (!originStartsWith(fixed, wsep))
-                    origin = prep->raw_apis.end();
-                else if (originStartsWith(path, wsep))
-                    break;
-                else
-                    ++origin;
-            }
-
-            if (origin == prep->raw_apis.end())
-                break;
-
-            ++start_new;
         }
+        else
+        {
+            QString fixed = *origin;
+            fixed.truncate(origin_len);
+
+            path = fixed;
+
+            while (start_new < end_new)
+            {
+                // Add this word to the current path.
+                path.append(wsep);
+                path.append(new_context[start_new]);
+                origin_len = path.length();
+
+                // Skip entries in the current origin that don't match the
+                // path.
+                while (origin != prep->raw_apis.end())
+                {
+                    // See if the current origin has come to an end.
+                    if (!originStartsWith(fixed, wsep))
+                        origin = prep->raw_apis.end();
+                    else if (originStartsWith(path, wsep))
+                        break;
+                    else
+                        ++origin;
+                }
+
+                if (origin == prep->raw_apis.end())
+                    break;
+
+                ++start_new;
+            }
+        }
+
+        // Terminate the path.
+        path.append(wsep);
 
         // If the new text wasn't recognised then reset the origin.
         if (origin == prep->raw_apis.end())
@@ -448,7 +450,7 @@ bool QsciAPIs::originStartsWith(const QString &path, const QString &wsep)
     // follows in the origin is either a word separator or a (.
     QString tail = orig.mid(path.length());
 
-    return (tail.startsWith(wsep) || tail.at(0) == '(');
+    return (!tail.isEmpty() && (tail.startsWith(wsep) || tail.at(0) == '('));
 }
 
 
@@ -831,9 +833,9 @@ void QsciAPIs::cancelPreparation()
 
 
 // Check that a prepared API file exists.
-bool QsciAPIs::isPrepared(const QString &fname) const
+bool QsciAPIs::isPrepared(const QString &filename) const
 {
-    QString pname = prepName(fname);
+    QString pname = prepName(filename);
 
     if (pname.isEmpty())
         return false;
@@ -845,9 +847,9 @@ bool QsciAPIs::isPrepared(const QString &fname) const
 
 
 // Load the prepared API information.
-bool QsciAPIs::loadPrepared(const QString &fname)
+bool QsciAPIs::loadPrepared(const QString &filename)
 {
-    QString pname = prepName(fname);
+    QString pname = prepName(filename);
 
     if (pname.isEmpty())
         return false;
@@ -920,9 +922,9 @@ bool QsciAPIs::loadPrepared(const QString &fname)
 
 
 // Save the prepared API information.
-bool QsciAPIs::savePrepared(const QString &fname) const
+bool QsciAPIs::savePrepared(const QString &filename) const
 {
-    QString pname = prepName(fname, true);
+    QString pname = prepName(filename, true);
 
     if (pname.isEmpty())
         return false;
@@ -963,11 +965,11 @@ QString QsciAPIs::defaultPreparedName() const
 
 
 // Return the name of a prepared API file.
-QString QsciAPIs::prepName(const QString &fname, bool mkpath) const
+QString QsciAPIs::prepName(const QString &filename, bool mkpath) const
 {
     // Handle the tivial case.
-    if (!fname.isEmpty())
-        return fname;
+    if (!filename.isEmpty())
+        return filename;
 
     QString pdname;
     char *qsci = getenv("QSCIDIR");
@@ -999,7 +1001,7 @@ QStringList QsciAPIs::installedAPIFiles() const
         return QStringList();
 
     QDir apidir = QDir(QString("%1/qsci/api/%2").arg(qtdir).arg(lexer()->lexer()));
-    QStringList fnames;
+    QStringList filenames;
 
     const QFileInfoList *flist = apidir.entryInfoList("*.api", QDir::Files, QDir::IgnoreCase);
 
@@ -1010,10 +1012,10 @@ QStringList QsciAPIs::installedAPIFiles() const
 
         while ((fi = it.current()) != 0)
         {
-            fnames << fi->absFilePath();
+            filenames << fi->absFilePath();
             ++it;
         }
     }
 
-    return fnames;
+    return filenames;
 }
