@@ -1,6 +1,6 @@
 // This module implements the QsciCommandSet class.
 //
-// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -35,7 +35,7 @@
 // Starting with QScintilla v2.7 the standard OS/X keyboard shortcuts are used
 // where possible.  In order to restore the behaviour of earlier versions then
 // #define DONT_USE_OSX_KEYS here or add it to the qmake project (.pro) file.
-#if defined(Q_WS_MAC) && !defined(DONT_USE_OSX_KEYS)
+#if defined(Q_OS_MAC) && !defined(DONT_USE_OSX_KEYS)
 #define USING_OSX_KEYS
 #else
 #undef  USING_OSX_KEYS
@@ -970,7 +970,7 @@ QsciCommand *QsciCommandSet::boundTo(int key) const
 {
     for (int i = 0; i < cmds.count(); ++i)
     {
-        QsciCommand *cmd = cmds.at(i);
+        QsciCommand *cmd = const_cast<QsciCommandSet *>(this)->cmds.at(i);
 
         if (cmd->key() == key || cmd->alternateKey() == key)
             return cmd;
@@ -985,7 +985,7 @@ QsciCommand *QsciCommandSet::find(QsciCommand::Command command) const
 {
     for (int i = 0; i < cmds.count(); ++i)
     {
-        QsciCommand *cmd = cmds.at(i);
+        QsciCommand *cmd = const_cast<QsciCommandSet *>(this)->cmds.at(i);
 
         if (cmd->command() == command)
             return cmd;
