@@ -1,6 +1,6 @@
 // This module implements the QsciLexerCPP class.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -41,6 +36,7 @@ QsciLexerCPP::QsciLexerCPP(QObject *parent, const char *name,
     : QsciLexer(parent, name),
       fold_atelse(false), fold_comments(false), fold_compact(true),
       fold_preproc(true), style_preproc(false), dollars(true),
+      highlight_triple(false), highlight_hash(false),
       nocase(caseInsensitiveKeywords)
 {
 }
@@ -129,44 +125,94 @@ QColor QsciLexerCPP::defaultColor(int style) const
     switch (style)
     {
     case Default:
-        return QColor(0x80,0x80,0x80);
+        return QColor(0x80, 0x80, 0x80);
 
     case Comment:
     case CommentLine:
-        return QColor(0x00,0x7f,0x00);
+        return QColor(0x00, 0x7f, 0x00);
 
     case CommentDoc:
     case CommentLineDoc:
-        return QColor(0x3f,0x70,0x3f);
+        return QColor(0x3f, 0x70, 0x3f);
 
     case Number:
-        return QColor(0x00,0x7f,0x7f);
+        return QColor(0x00, 0x7f, 0x7f);
 
     case Keyword:
-        return QColor(0x00,0x00,0x7f);
+        return QColor(0x00, 0x00, 0x7f);
 
     case DoubleQuotedString:
     case SingleQuotedString:
-        return QColor(0x7f,0x00,0x7f);
+    case RawString:
+        return QColor(0x7f, 0x00, 0x7f);
 
     case PreProcessor:
-        return QColor(0x7f,0x7f,0x00);
+        return QColor(0x7f, 0x7f, 0x00);
 
     case Operator:
     case UnclosedString:
-        return QColor(0x00,0x00,0x00);
+        return QColor(0x00, 0x00, 0x00);
 
-    case Identifier:
-        break;
+    case VerbatimString:
+    case TripleQuotedVerbatimString:
+    case HashQuotedString:
+        return QColor(0x00, 0x7f, 0x00);
 
     case Regex:
-        return QColor(0x3f,0x7f,0x3f);
+        return QColor(0x3f, 0x7f, 0x3f);
 
     case CommentDocKeyword:
-        return QColor(0x30,0x60,0xa0);
+        return QColor(0x30, 0x60, 0xa0);
 
     case CommentDocKeywordError:
-        return QColor(0x80,0x40,0x20);
+        return QColor(0x80, 0x40, 0x20);
+
+    case PreProcessorComment:
+        return QColor(0x65, 0x99, 0x00);
+
+    case InactiveDefault:
+    case InactiveUUID:
+    case InactiveCommentLineDoc:
+    case InactiveKeywordSet2:
+    case InactiveCommentDocKeyword:
+    case InactiveCommentDocKeywordError:
+        return QColor(0xc0, 0xc0, 0xc0);
+
+    case InactiveComment:
+    case InactiveCommentLine:
+    case InactiveNumber:
+    case InactiveVerbatimString:
+    case InactiveTripleQuotedVerbatimString:
+    case InactiveHashQuotedString:
+        return QColor(0x90, 0xb0, 0x90);
+
+    case InactiveCommentDoc:
+        return QColor(0xd0, 0xd0, 0xd0);
+
+    case InactiveKeyword:
+        return QColor(0x90, 0x90, 0xb0);
+
+    case InactiveDoubleQuotedString:
+    case InactiveSingleQuotedString:
+    case InactiveRawString:
+        return QColor(0xb0, 0x90, 0xb0);
+
+    case InactivePreProcessor:
+        return QColor(0xb0, 0xb0, 0x90);
+
+    case InactiveOperator:
+    case InactiveIdentifier:
+    case InactiveGlobalClass:
+        return QColor(0xb0, 0xb0, 0xb0);
+
+    case InactiveUnclosedString:
+        return QColor(0x00, 0x00, 0x00);
+
+    case InactiveRegex:
+        return QColor(0x7f, 0xaf, 0x7f);
+
+    case InactivePreProcessorComment:
+        return QColor(0xa0, 0xc0, 0x90);
     }
 
     return QsciLexer::defaultColor(style);
@@ -176,8 +222,20 @@ QColor QsciLexerCPP::defaultColor(int style) const
 // Returns the end-of-line fill for a style.
 bool QsciLexerCPP::defaultEolFill(int style) const
 {
-    if (style == UnclosedString)
+    switch (style)
+    {
+    case UnclosedString:
+    case InactiveUnclosedString:
+    case VerbatimString:
+    case InactiveVerbatimString:
+    case Regex:
+    case InactiveRegex:
+    case TripleQuotedVerbatimString:
+    case InactiveTripleQuotedVerbatimString:
+    case HashQuotedString:
+    case InactiveHashQuotedString:
         return true;
+    }
 
     return QsciLexer::defaultEolFill(style);
 }
@@ -191,29 +249,52 @@ QFont QsciLexerCPP::defaultFont(int style) const
     switch (style)
     {
     case Comment:
+    case InactiveComment:
     case CommentLine:
+    case InactiveCommentLine:
     case CommentDoc:
+    case InactiveCommentDoc:
     case CommentLineDoc:
+    case InactiveCommentLineDoc:
     case CommentDocKeyword:
+    case InactiveCommentDocKeyword:
     case CommentDocKeywordError:
+    case InactiveCommentDocKeywordError:
 #if defined(Q_OS_WIN)
         f = QFont("Comic Sans MS",9);
+#elif defined(Q_OS_MAC)
+        f = QFont("Comic Sans MS", 12);
 #else
         f = QFont("Bitstream Vera Serif",9);
 #endif
         break;
 
     case Keyword:
+    case InactiveKeyword:
     case Operator:
+    case InactiveOperator:
         f = QsciLexer::defaultFont(style);
         f.setBold(true);
         break;
 
     case DoubleQuotedString:
+    case InactiveDoubleQuotedString:
     case SingleQuotedString:
+    case InactiveSingleQuotedString:
     case UnclosedString:
+    case InactiveUnclosedString:
+    case VerbatimString:
+    case InactiveVerbatimString:
+    case Regex:
+    case InactiveRegex:
+    case TripleQuotedVerbatimString:
+    case InactiveTripleQuotedVerbatimString:
+    case HashQuotedString:
+    case InactiveHashQuotedString:
 #if defined(Q_OS_WIN)
         f = QFont("Courier New",10);
+#elif defined(Q_OS_MAC)
+        f = QFont("Courier", 12);
 #else
         f = QFont("Bitstream Vera Sans Mono",9);
 #endif
@@ -271,56 +352,128 @@ QString QsciLexerCPP::description(int style) const
     case Default:
         return tr("Default");
 
+    case InactiveDefault:
+        return tr("Inactive default");
+
     case Comment:
         return tr("C comment");
 
+    case InactiveComment:
+        return tr("Inactive C comment");
+
     case CommentLine:
         return tr("C++ comment");
 
+    case InactiveCommentLine:
+        return tr("Inactive C++ comment");
+
     case CommentDoc:
         return tr("JavaDoc style C comment");
 
+    case InactiveCommentDoc:
+        return tr("Inactive JavaDoc style C comment");
+
     case Number:
         return tr("Number");
 
+    case InactiveNumber:
+        return tr("Inactive number");
+
     case Keyword:
         return tr("Keyword");
 
+    case InactiveKeyword:
+        return tr("Inactive keyword");
+
     case DoubleQuotedString:
         return tr("Double-quoted string");
 
+    case InactiveDoubleQuotedString:
+        return tr("Inactive double-quoted string");
+
     case SingleQuotedString:
         return tr("Single-quoted string");
 
+    case InactiveSingleQuotedString:
+        return tr("Inactive single-quoted string");
+
     case UUID:
         return tr("IDL UUID");
 
+    case InactiveUUID:
+        return tr("Inactive IDL UUID");
+
     case PreProcessor:
         return tr("Pre-processor block");
 
+    case InactivePreProcessor:
+        return tr("Inactive pre-processor block");
+
     case Operator:
         return tr("Operator");
 
+    case InactiveOperator:
+        return tr("Inactive operator");
+
     case Identifier:
         return tr("Identifier");
 
+    case InactiveIdentifier:
+        return tr("Inactive identifier");
+
     case UnclosedString:
         return tr("Unclosed string");
 
+    case InactiveUnclosedString:
+        return tr("Inactive unclosed string");
+
+    case VerbatimString:
+        return tr("C# verbatim string");
+
+    case InactiveVerbatimString:
+        return tr("Inactive C# verbatim string");
+
+    case Regex:
+        return tr("JavaScript regular expression");
+
+    case InactiveRegex:
+        return tr("Inactive JavaScript regular expression");
+
     case CommentLineDoc:
         return tr("JavaDoc style C++ comment");
 
+    case InactiveCommentLineDoc:
+        return tr("Inactive JavaDoc style C++ comment");
+
     case KeywordSet2:
         return tr("Secondary keywords and identifiers");
 
+    case InactiveKeywordSet2:
+        return tr("Inactive secondary keywords and identifiers");
+
     case CommentDocKeyword:
         return tr("JavaDoc keyword");
 
+    case InactiveCommentDocKeyword:
+        return tr("Inactive JavaDoc keyword");
+
     case CommentDocKeywordError:
         return tr("JavaDoc keyword error");
 
+    case InactiveCommentDocKeywordError:
+        return tr("Inactive JavaDoc keyword error");
+
     case GlobalClass:
         return tr("Global classes and typedefs");
+
+    case InactiveGlobalClass:
+        return tr("Inactive global classes and typedefs");
+
+    case RawString:
+        return tr("C++ raw string");
+
+    case InactiveRawString:
+        return tr("Inactive C++ raw string");
     }
 
     return QString();
@@ -330,9 +483,31 @@ QString QsciLexerCPP::description(int style) const
 // Returns the background colour of the text for a style.
 QColor QsciLexerCPP::defaultPaper(int style) const
 {
-    if (style == UnclosedString)
+    switch (style)
+    {
+    case UnclosedString:
+    case InactiveUnclosedString:
         return QColor(0xe0,0xc0,0xe0);
 
+    case VerbatimString:
+    case InactiveVerbatimString:
+    case TripleQuotedVerbatimString:
+    case InactiveTripleQuotedVerbatimString:
+        return QColor(0xe0,0xff,0xe0);
+
+    case Regex:
+    case InactiveRegex:
+        return QColor(0xe0,0xf0,0xe0);
+
+    case RawString:
+    case InactiveRawString:
+        return QColor(0xff,0xf3,0xff);
+
+    case HashQuotedString:
+    case InactiveHashQuotedString:
+        return QColor(0xe7,0xff,0xd7);
+    }
+
     return QsciLexer::defaultPaper(style);
 }
 
@@ -346,6 +521,8 @@ void QsciLexerCPP::refreshProperties()
     setPreprocProp();
     setStylePreprocProp();
     setDollarsProp();
+    setHighlightTripleProp();
+    setHighlightHashProp();
 }
 
 
@@ -398,6 +575,20 @@ bool QsciLexerCPP::readProperties(QSettings &qs,const QString &prefix)
     else
         rc = false;
 
+    flag = qs.readBoolEntry(prefix + "highlighttriple", false, &ok);
+
+    if (ok)
+        highlight_triple = flag;
+    else
+        rc = false;
+
+    flag = qs.readBoolEntry(prefix + "highlighthash", false, &ok);
+
+    if (ok)
+        highlight_hash = flag;
+    else
+        rc = false;
+
     return rc;
 }
 
@@ -425,17 +616,16 @@ bool QsciLexerCPP::writeProperties(QSettings &qs,const QString &prefix) const
     if (!qs.writeEntry(prefix + "dollars", dollars))
         rc = false;
 
+    if (!qs.writeEntry(prefix + "highlighttriple", highlight_triple))
+        rc = false;
+
+    if (!qs.writeEntry(prefix + "highlighthash", highlight_hash))
+        rc = false;
+
     return rc;
 }
 
 
-// Return true if else can be folded.
-bool QsciLexerCPP::foldAtElse() const
-{
-    return fold_atelse;
-}
-
-
 // Set if else can be folded.
 void QsciLexerCPP::setFoldAtElse(bool fold)
 {
@@ -452,13 +642,6 @@ void QsciLexerCPP::setAtElseProp()
 }
 
 
-// Return true if comments can be folded.
-bool QsciLexerCPP::foldComments() const
-{
-    return fold_comments;
-}
-
-
 // Set if comments can be folded.
 void QsciLexerCPP::setFoldComments(bool fold)
 {
@@ -475,13 +658,6 @@ void QsciLexerCPP::setCommentProp()
 }
 
 
-// Return true if folds are compact.
-bool QsciLexerCPP::foldCompact() const
-{
-    return fold_compact;
-}
-
-
 // Set if folds are compact
 void QsciLexerCPP::setFoldCompact(bool fold)
 {
@@ -498,13 +674,6 @@ void QsciLexerCPP::setCompactProp()
 }
 
 
-// Return true if preprocessor blocks can be folded.
-bool QsciLexerCPP::foldPreprocessor() const
-{
-    return fold_preproc;
-}
-
-
 // Set if preprocessor blocks can be folded.
 void QsciLexerCPP::setFoldPreprocessor(bool fold)
 {
@@ -521,13 +690,6 @@ void QsciLexerCPP::setPreprocProp()
 }
 
 
-// Return true if preprocessor lines are styled.
-bool QsciLexerCPP::stylePreprocessor() const
-{
-    return style_preproc;
-}
-
-
 // Set if preprocessor lines are styled.
 void QsciLexerCPP::setStylePreprocessor(bool style)
 {
@@ -544,13 +706,6 @@ void QsciLexerCPP::setStylePreprocProp()
 }
 
 
-// Return true if '$' characters are allowed.
-bool QsciLexerCPP::dollarsAllowed() const
-{
-    return dollars;
-}
-
-
 // Set if '$' characters are allowed.
 void QsciLexerCPP::setDollarsAllowed(bool allowed)
 {
@@ -565,3 +720,37 @@ void QsciLexerCPP::setDollarsProp()
 {
     emit propertyChanged("lexer.cpp.allow.dollars",(dollars ? "1" : "0"));
 }
+
+
+// Set if triple quoted strings are highlighted.
+void QsciLexerCPP::setHighlightTripleQuotedStrings(bool enabled)
+{
+    highlight_triple = enabled;
+
+    setHighlightTripleProp();
+}
+
+
+// Set the "lexer.cpp.triplequoted.strings" property.
+void QsciLexerCPP::setHighlightTripleProp()
+{
+    emit propertyChanged("lexer.cpp.triplequoted.strings",
+            (highlight_triple ? "1" : "0"));
+}
+
+
+// Set if hash quoted strings are highlighted.
+void QsciLexerCPP::setHighlightHashQuotedStrings(bool enabled)
+{
+    highlight_hash = enabled;
+
+    setHighlightHashProp();
+}
+
+
+// Set the "lexer.cpp.hashquoted.strings" property.
+void QsciLexerCPP::setHighlightHashProp()
+{
+    emit propertyChanged("lexer.cpp.hashquoted.strings",
+            (highlight_hash ? "1" : "0"));
+}
