@@ -639,10 +639,10 @@ void QsciScintillaBase::mousePressEvent(QMouseEvent *e)
         // the modifier to be configured.
         bool shift = e->state() & Qt::ShiftButton;
         bool ctrl = e->state() & Qt::ControlButton;
-#if defined(Q_WS_X11)
-        bool alt = ctrl;
-#else
+#if !defined(Q_WS_X11)
         bool alt = e->modifiers() & Qt::AltButton;
+#else
+        bool alt = ctrl;
 #endif
 
         sci->ButtonDown(pt, clickTime, shift, ctrl, alt);
